@@ -8,7 +8,9 @@
 import './browser/polyfills';
 
 /* common libs */
+import * as cancel from './common/cancel';
 import * as platform from './common/platform';
+import * as promise from './common/promise';
 import * as resource_safety from './common/resource_safety';
 
 /* browser specific libs */
@@ -23,6 +25,7 @@ import { ICrtError, CrtError } from './browser/error';
 
 export {
     auth,
+    cancel,
     crypto,
     http,
     io,
@@ -30,6 +33,7 @@ export {
     mqtt,
     mqtt5,
     platform,
+    promise,
     resource_safety,
     ICrtError,
     CrtError
