@@ -247,16 +247,26 @@ export class AwsIotMqttConnectionConfigBuilder {
      * @param username The username to use with the custom authorizer. If an empty string is passed, it will
      *                 check to see if a username has already been set (via WithUsername function). If no
      *                 username is set then no username will be passed with the MQTT connection.
-     * @param authorizerName The name of the custom authorizer. If an empty string is passed, then
+     * @param authorizer_name The name of the custom authorizer. If an empty string is passed, then
      *                       'x-amz-customauthorizer-name' will not be added with the MQTT connection.
-     * @param authorizerSignature The signature of the custom authorizer. If an empty string is passed, then
+     * @param authorizer_signature The signature of the custom authorizer. If an empty string is passed, then
      *                            'x-amz-customauthorizer-signature' will not be added with the MQTT connection.
+     *                            The signature must be based on the private key associated with the custom authorizer.
+     *                            The signature must be base64 encoded.
+     *                            Required if the custom authorizer has signing enabled.  It is strongly suggested to URL-encode
+     *                            this value; the SDK will not do so for you.
      * @param password The password to use with the custom authorizer. If null is passed, then no password will
      *                 be set.
+     * @param token_key_name Key used to extract the custom authorizer token from MQTT username query-string properties.
+     *                       Required if the custom authorizer has signing enabled.  It is strongly suggested to URL-encode
+     *                       this value; the SDK will not do so for you.
+     * @param token_value An opaque token value.
+     *                    Required if the custom authorizer has signing enabled. This value must be signed by the private
+     *                    key associated with the custom authorizer and the result placed in the authorizer_signature argument.
      */
-    with_custom_authorizer(username : string, authorizer_name : string, authorizer_signature : string, password : string) {
+    with_custom_authorizer(username : string, authorizer_name : string, authorizer_signature : string, password : string, token_key_name? : string, token_value? : string) {
         let username_string = iot_shared.populate_username_string_with_custom_authorizer(
-            "", username, authorizer_name, authorizer_signature, this.params.username);
+            "", username, authorizer_name, authorizer_signature, this.params.username, token_key_name, token_value);
         this.params.username = username_string;
         this.params.password = password;
         // Tells the websocket connection we are using a custom authorizer
