@@ -6,7 +6,9 @@
 // This is the entry point for the AWS CRT nodejs native libraries
 
 /* common libs */
+import * as cancel from './common/cancel';
 import * as platform from './common/platform';
+import * as promise from './common/promise';
 import * as resource_safety from './common/resource_safety';
 
 /* node specific libs */
@@ -24,6 +26,7 @@ import { ICrtError, CrtError } from './native/error';
 
 export {
     auth,
+    cancel,
     checksums,
     crypto,
     crt,
@@ -34,6 +37,7 @@ export {
     mqtt,
     mqtt5,
     platform,
+    promise,
     resource_safety,
     ICrtError,
     CrtError
